/*
 * LinShare is an open source filesharing software, part of the LinPKI software
 * suite, developed by Linagora.
 * 
 * Copyright (C) 2015 LINAGORA
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version, provided you comply with the Additional Terms applicable for
 * LinShare software by Linagora pursuant to Section 7 of the GNU Affero General
 * Public License, subsections (b), (c), and (e), pursuant to which you must
 * notably (i) retain the display of the “LinShare™” trademark/logo at the top
 * of the interface window, the display of the “You are using the Open Source
 * and free version of LinShare™, powered by Linagora © 2009–2015. Contribute to
 * Linshare R&D by subscribing to an Enterprise offer!” infobox and in the
 * e-mails sent with the Program, (ii) retain all hypertext links between
 * LinShare and linshare.org, between linagora.com and Linagora, and (iii)
 * refrain from infringing Linagora intellectual property rights over its
 * trademarks and commercial brands. Other Additional Terms apply, see
 * <http://www.linagora.com/licenses/> for more details.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License and
 * its applicable Additional Terms for LinShare along with this program. If not,
 * see <http://www.gnu.org/licenses/> for the GNU Affero General Public License
 * version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
 * applicable to LinShare software.
 */
package org.linagora.linshare.core.domain.vo;

import java.io.Serializable;

import org.linagora.linshare.core.domain.constants.FileSizeUnit;
import org.linagora.linshare.core.domain.constants.TimeUnit;
import org.linagora.linshare.core.domain.entities.ShareExpiryRule;

/** Defines a share expiry rule.
 */
public class ShareExpiryRuleVo implements Serializable {

	private static final long serialVersionUID = -896190348275893760L;
	private final Integer shareExpiryTime;
    private final TimeUnit shareExpiryUnit;
    private final Integer shareSize;
    private final FileSizeUnit shareSizeUnit;

    public ShareExpiryRuleVo(Integer shareExpiryTime, TimeUnit shareExpiryUnit,
        Integer shareSize, FileSizeUnit shareSizeUnit) {
        this.shareExpiryTime = shareExpiryTime;
        this.shareExpiryUnit = shareExpiryUnit;
        this.shareSize = shareSize;
        this.shareSizeUnit = shareSizeUnit;
    }

    public ShareExpiryRuleVo(ShareExpiryRule shareExpiryRule) {
        this.shareExpiryTime = shareExpiryRule.getShareExpiryTime();
        this.shareExpiryUnit = shareExpiryRule.getShareExpiryUnit();
        this.shareSize = shareExpiryRule.getShareSize();
        this.shareSizeUnit = shareExpiryRule.getShareSizeUnit();
    }

    public Integer getShareExpiryTime() {
        return shareExpiryTime;
    }

    public TimeUnit getShareExpiryUnit() {
        return shareExpiryUnit;
    }

    public Integer getShareSize() {
        return shareSize;
    }

    public FileSizeUnit getShareSizeUnit() {
        return shareSizeUnit;
    }

    public ShareExpiryRule getShareExpiryRule() {
        ShareExpiryRule shareExpiryRule = new ShareExpiryRule();
        shareExpiryRule.setShareExpiryTime(shareExpiryTime);
        shareExpiryRule.setShareExpiryUnit(shareExpiryUnit);
        shareExpiryRule.setShareSize(shareSize);
        shareExpiryRule.setShareSizeUnit(shareSizeUnit);
        return shareExpiryRule;
    }

}
